package com.example.batterylevelapp

import android.content.*
import android.os.BatteryManager
import android.os.Build
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.PluginRegistry.Registrar

class BatteryPlugin : MethodCallHandler, EventChannel.StreamHandler, FlutterPlugin {
    private var applicationContext: Context? = null
    private var chargingStateChangeReceiver: BroadcastReceiver? = null
    private var methodChannel: MethodChannel? = null
    private var eventChannel: EventChannel? = null
    override fun onAttachedToEngine(binding: FlutterPluginBinding) {
        onAttachedToEngine(binding.applicationContext, binding.binaryMessenger)
    }

    private fun onAttachedToEngine(applicationContext: Context, messenger: BinaryMessenger) {
        this.applicationContext = applicationContext
        methodChannel = MethodChannel(messenger, "plugins.flutter.io/battery")
        eventChannel = EventChannel(messenger, "plugins.flutter.io/charging")
        eventChannel!!.setStreamHandler(this)
        methodChannel!!.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPluginBinding) {
        applicationContext = null
        methodChannel!!.setMethodCallHandler(null)
        methodChannel = null
        eventChannel!!.setStreamHandler(null)
        eventChannel = null
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        if (call.method == "getBatteryLevel") {
            val batteryLevel = batteryLevel
            if (batteryLevel != -1) {
                result.success(batteryLevel)
            } else {
                result.error("UNAVAILABLE", "Battery level not available.", null)
            }
        } else {
            result.notImplemented()
        }
    }

    override fun onListen(arguments: Any, events: EventSink) {
        chargingStateChangeReceiver = createChargingStateChangeReceiver(events)
        applicationContext!!.registerReceiver(
                chargingStateChangeReceiver, IntentFilter(Intent.ACTION_BATTERY_CHANGED))
    }

    override fun onCancel(arguments: Any) {
        applicationContext!!.unregisterReceiver(chargingStateChangeReceiver)
        chargingStateChangeReceiver = null
    }

    private val batteryLevel: Int
        private get() {
            var batteryLevel = -1
            batteryLevel = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                val batteryManager = applicationContext!!.getSystemService(Context.BATTERY_SERVICE) as BatteryManager
                batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)
            } else {
                val intent = ContextWrapper(applicationContext)
                        .registerReceiver(null, IntentFilter(Intent.ACTION_BATTERY_CHANGED))
                (intent.getIntExtra(BatteryManager.EXTRA_LEVEL, -1) * 100
                        / intent.getIntExtra(BatteryManager.EXTRA_SCALE, -1))
            }
            return batteryLevel
        }

    private fun createChargingStateChangeReceiver(events: EventSink): BroadcastReceiver {
        return object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                val status = intent.getIntExtra(BatteryManager.EXTRA_STATUS, -1)
                when (status) {
                    BatteryManager.BATTERY_STATUS_CHARGING -> events.success("charging")
                    BatteryManager.BATTERY_STATUS_FULL -> events.success("full")
                    BatteryManager.BATTERY_STATUS_DISCHARGING -> events.success("discharging")
                    else -> events.error("UNAVAILABLE", "Charging status unavailable", null)
                }
            }
        }
    }

    companion object {
        /**
         * Plugin registration.
         */
        fun registerWith(registrar: Registrar) {
            val instance = BatteryPlugin()
            instance.onAttachedToEngine(registrar.context(), registrar.messenger())
        }
    }
}